from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from .models import Coil
from suppliers.models import Supplier
from colors.models import Color
from django.db import IntegrityError

@login_required
def coils(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            coils = Coil.objects.all()
            return render(request, "inventory/coil/coils.html", {'coils': coils})
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect('inventory:coils')
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def add_coil(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        if request.method == 'POST':
            try:
                coil_number = request.POST.get('coil_number')
                gauge = request.POST.get('gauge')
                color_id = request.POST.get('color')
                finish = request.POST.get('finish')
                gross_weight = request.POST.get('gross_weight')
                net_weight = request.POST.get('net_weight')
                meters = request.POST.get('meters')
                supplier_id = request.POST.get('supplier')
                added_by = request.user

                color = get_object_or_404(Color, id=color_id)
                supplier = get_object_or_404(Supplier, supplier_id=supplier_id)

                coil = Coil.objects.create(
                    coil_number=coil_number,
                    gauge=gauge,
                    color=color,
                    finish=finish,
                    gross_weight=gross_weight,
                    net_weight=net_weight,
                    meters=meters,
                    supplier=supplier,
                    added_by=added_by
                )
                messages.success(request, 'Coil added successfully.')
                return redirect('inventory:coils')
            except IntegrityError as e:
                messages.error(request, f"Data integrity error: {str(e)}")
            except Exception as e:
                messages.error(request, f"An error occurred: {str(e)}")

        try:
            colors = Color.objects.all()
            suppliers = Supplier.objects.all()
            return render(request, "inventory/coil/add_coil.html", {'colors': colors, 'suppliers': suppliers})
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect('inventory:coils')
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def edit_coil(request, coil_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            coil = get_object_or_404(Coil, id=coil_id)
            if request.method == 'POST':
                coil.coil_number = request.POST.get('coil_number')
                coil.gauge = request.POST.get('gauge')
                coil.color_id = request.POST.get('color')
                coil.finish = request.POST.get('finish')
                coil.gross_weight = request.POST.get('gross_weight')
                coil.net_weight = request.POST.get('net_weight')
                coil.meters = request.POST.get('meters')
                coil.supplier_id = request.POST.get('supplier')

                coil.save()
                messages.success(request, 'Coil updated successfully.')
                return redirect('inventory:coils')

            colors = Color.objects.all()
            suppliers = Supplier.objects.all()
            return render(request, "inventory/coil/edit_coil.html", {'coil': coil, 'colors': colors, 'suppliers': suppliers})
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect('inventory:coils')
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def delete_coil(request, coil_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            coil = get_object_or_404(Coil, id=coil_id)
            coil_number = coil.coil_number
            coil.delete()
            messages.success(request, f"Coil {coil_number} has been deleted successfully.")
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
        return redirect("inventory:coils")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")